r"""
 __  __                           _
|  \/  | ___ _ __ ___   ___  _ __(_)
| |\/| |/ _ \ '_ ` _ \ / _ \| '__| |
| |  | |  __/ | | | | | (_) | |  | |
|_|  |_|\___|_| |_| |_|\___/|_|  |_|
                  perfectam memoriam
                       memorilabs.ai
"""

ATHROPIC_LLM_PROVIDER = "anthropic"
GOOGLE_LLM_PROVIDER = "google"
LANGCHAIN_CHATBEDROCK_LLM_PROVIDER = "chatbedrock"
LANGCHAIN_CHATGOOGLEGENAI_LLM_PROVIDER = "chatgooglegenai"
LANGCHAIN_CHATVERTEXAI_LLM_PROVIDER = "chatvertexai"
LANGCHAIN_FRAMEWORK_PROVIDER = "langchain"
LANGCHAIN_OPENAI_LLM_PROVIDER = "chatopenai"
OPENAI_LLM_PROVIDER = "openai"
PYDANTIC_AI_FRAMEWORK_PROVIDER = "pydantic_ai"
PYDANTIC_AI_OPENAI_LLM_PROVIDER = "openai"
XAI_LLM_PROVIDER = "xai"
